//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooChebychev.h"
#include "RooExponential.h"
#include "RooBifurGauss.h"
#include "RooAddModel.h"
#include "RooProdPdf.h"
#include "TCanvas.h"
#include "RooPlot.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "TLegend.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "RooJohnson.h"
#include "myRooPolBG.cpp"

using namespace RooFit ;
using namespace std;

void deltaM_kspipi()
{
  TFile *f = new TFile("component_wise_trees_for_fit.root");

  TTree *tree = (TTree*)f->Get("peaking");

  cout << "Entries = " << tree->GetEntries() << endl;
  
  RooRealVar deltaM_kspipi("deltaM_kspipi","", 0.139, 0.159, "");
  RooDataSet data("data","data", tree, RooArgSet(deltaM_kspipi));
  
  
  //signal
  RooRealVar fG("f_{G}", "",0.2, 0, 1);//0.13, 0.26,0.30, 0.35, 0.36, 0.38,

  RooRealVar mean_G("#mu", "mean",0.14, 0.139, 0.159);
  RooRealVar sigma_G("#sigma_{G}", "sigma", 0.009, 0, 1);
  RooGaussian gauss("gauss", "gauss component", deltaM_kspipi, mean_G, sigma_G);
 
  RooRealVar mean("#mu_{J}", "mean",0.14, 0.139, 0.159);
  RooRealVar lambda("#lambda_{J}", "sigma", 0.009, 0, 1);
  RooRealVar delta("#delta_{J}", "delta", 1.0,-2,2);
  RooRealVar gamma("#gamma_{J}", "gamma", 0.111,-5,5);//0.112,0.11
  RooJohnson johnson("johnson", "johnson component", deltaM_kspipi, mean_G, lambda, delta, gamma);
  //RooJohnson sig("sig", "johnson component", deltaM_kspipi, mean, lambda, delta, gamma);

  RooAddPdf sig("sig" ,"gauss + johnson",gauss, johnson,fG);
  
  //background
  RooRealVar mpi("mpi", "nominal pion mass", 0.13957039);
  //RooRealVar alpha("#alpha", "alpha", 20, -100., 100.);
  RooRealVar alpha("#alpha", "alpha", 0.);

  RooRealVar beta("#beta", "beta", 0.);
  myRooPolBG bkg("bkg","background component", deltaM_kspipi, mpi, alpha, beta);
  
  //total
  RooRealVar n_sig("N_{sig}", "n_{s}", 1000, 1, 100000);
  RooRealVar n_bkg("N_{bkg}", "n_{b}", 0);
  RooAddPdf pdf("pdf", "two component model",RooArgList(sig, bkg), RooArgList(n_sig, n_bkg));
  
  //fit
  RooFitResult *fitresult = pdf.fitTo(data, Save(true), Strategy(2), Extended(true));
  
  RooPlot* deltaMplot = deltaM_kspipi.frame();
  data.plotOn(deltaMplot);
  pdf.plotOn(deltaMplot);
  pdf.paramOn(deltaMplot);
  deltaMplot->SetYTitle("Candidates per 0.2 MeV/#it{c}^{2}");
  deltaMplot->GetXaxis()->SetLabelOffset(999);
  double nbins = 5*deltaMplot->chiSquare(5) / (deltaMplot->chiSquare(5) - deltaMplot->chiSquare());// five is the no. of fit parameters
  cout << "chi^2/ndf = " << deltaMplot->chiSquare()*nbins << "/" << nbins-5 << endl;

  double deltaMplot_chi2 = deltaMplot->chiSquare();
  TPaveLabel *t1 = new TPaveLabel(0.7,0.6,0.9,0.66, Form("#chi^{2}/ndf = %f", deltaMplot_chi2),"brNDC");
  t1->SetBorderSize(1);
  t1->SetFillStyle(0);
  //t1->SetTextFont(40);
  t1->SetTextSize(0.70);
  t1->SetLineColor(0);

  TLegend *leg = new TLegend(0.57,0.73,0.78,0.93);
  leg->SetBorderSize(0);  // no border
  leg->SetFillStyle(0);
  leg->AddEntry("deltaM_kspipi","Data","pe");
  leg->AddEntry("pdf","Total Fit","l");
  leg->AddEntry("gauss","Gaussian","l");
  leg->AddEntry("johnson","Johnson","l");
  
  RooHist* hpull = deltaMplot->pullHist();
  hpull->SetFillStyle(1001);
  hpull->SetFillColor(1);
  for(int i=0;i<hpull->GetN();++i) hpull->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullplot = deltaM_kspipi.frame(Title(" "));
  pullplot->addPlotable(hpull,"B");
  // pullplot->SetYTitle("Pull");
  pullplot->SetXTitle("#it{#Deltam} [GeV/c^{2}]");
  pullplot->SetMinimum(-4.);
  pullplot->SetMaximum(4.);
  pullplot->GetXaxis()->SetLabelSize(0.1);
  pullplot->GetXaxis()->SetTitleSize(0.1);
  pullplot->GetYaxis()->SetLabelSize(0.07);
  
  
  TCanvas *canvas = new TCanvas("canvas","canvas", 600, 600);
 
  Double_t xlow, ylow, xup, yup;
  canvas->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  canvas->Divide(1,2);

  TVirtualPad *upPad = canvas->GetPad(1);
  upPad->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  
  TVirtualPad *dwPad = canvas->GetPad(2);
  dwPad->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  
  canvas->Update();
  canvas->cd(1);
  leg->Draw();
  pdf.plotOn(deltaMplot,Components(RooArgSet(gauss)),LineColor(kRed),LineStyle(kDashed));
  pdf.plotOn(deltaMplot,Components(RooArgSet(johnson)),LineColor(kGreen),LineStyle(kDashed));
  deltaMplot->Draw();
  t1->Draw();
  leg->DrawClone();


  canvas->cd(2);
  pullplot->Draw();

  
}
